@@ -16,14 +16,14 @@ test_that("resolve_palette works", {
 
 test_that("colour_with_palette works for nums and strs", {
 
-  res <- spatialwidget:::rcpp_colour_str_with_palette("viridis", letters, 255.0, "#808080FF", TRUE)
+  res <- spatialwidget:::rcpp_colour_str_with_palette("viridis", letters, 255.0, "#808080FF", TRUE, "fill_colour" )
   col <- colourvalues::color_values(letters, summary = T)
 
   expect_equal( res$colours, col$colours )
   expect_equal( res$summary_colours, col$summary_colours )
   expect_equal( res$summary_values, col$summary_values )
 
-  res <- spatialwidget:::rcpp_colour_num_with_palette("viridis", 1:26, 255.0, "#808080FF", TRUE)
+  res <- spatialwidget:::rcpp_colour_num_with_palette("viridis", 1:26, 255.0, "#808080FF", TRUE, "fill_colour" )
   col <- colourvalues::color_values(1:26, n_summaries = 5, format = T)
 
   expect_equal( res$colours, col$colours )
